@@ -24,7 +24,9 @@
 
 import sys
 
-from TTcrossGenzPlotData import plot_test
+# from TTcrossGenzPlotData import plot_test
+# from TTdmrgGenzPlotData import plot_test
+from QTTdmrgGenzPlotData import plot_test
 
 def main(argv):
     Ns_FUNC = range(6)
