@@ -38,7 +38,7 @@ import TensorToolbox.multilinalg as mla
 
 from SpectralToolbox import Spectral1D as S1D
 from SpectralToolbox import SpectralND as SND
-from UQToolbox import UncertaintyQuantification as UQ
+from UQToolbox import RandomSampling as RS
 
 import matplotlib.pyplot as plt
 from matplotlib import cm
@@ -60,7 +60,7 @@ npr.seed(1)
 
 TESTS = [3]
 IS_PLOTTING = True
-TITLE = False
+TITLE = True
 fsize = (6,4.5)
 
 if 1 in TESTS:
@@ -211,9 +211,9 @@ if 3 in TESTS:
     eps = 1e-13
 
     xspan = [-1.,1.]
-    d = 2
+    d = 4
     size1D = 20
-    nzdim = [0,1]
+    nzdim = [2,3]
     vol = (xspan[1]-xspan[0])**d
     
     # # Using simplex
@@ -294,7 +294,7 @@ if 3 in TESTS:
         for idxs,coeff in zip(params['idxs'],params['coeffs']):
             tmp = 1.
             for i,ii in enumerate(idxs):
-                tmp *= params['VV'][i][:,ii] # * params['Gammas'][i][ii]
+                tmp *= params['VV'][nzdim[i]][:,ii] # * params['Gammas'][i][ii]
             # for i,ii in enumerate(idx): tmp *= P.GradEvaluate(np.array([ X[nzdim[i]] ]), ii, 0, norm=True)
 
             out += coeff * tmp / ( len(nzdim) * np.dot( w, VV[0][:,0]))
@@ -308,7 +308,7 @@ if 3 in TESTS:
                'Gammas': Gammas}
     TW = DT.TensorWrapper( f, X, params )
     
-    TTapprox = DT.TTvec(TW,eps=eps,method='ttcross',lr_delta=delta,mv_eps=maxvoleps)
+    TTapprox = DT.TTvec(TW,eps=eps,method='ttcross',delta=delta,mv_eps=maxvoleps)
 
     print "TTcross: grid size %d" % TW.get_size()
     print "TTcross: function evaluations %d" % TW.get_fill_level()
@@ -338,12 +338,12 @@ if 3 in TESTS:
     MCstep = 10000
     var = 1.
     dist = stats.uniform()
-    DIST = UQ.MultiDimDistribution([dist] * d)
+    DIST = RS.MultiDimDistribution([dist] * d)
     intf = []
     values = []
     while (len(values) < MCestMinIter or var > mean**2. * MCestVarLimit) and len(values) < MCestMaxIter:
         # Monte Carlo
-        xx = DIST.rvs(MCstep)
+        xx = np.asarray( DIST.rvs(MCstep) )
 
         VsI = None
         VsI = [P.GradVandermonde1D(xx[:,i]*2.-1.,size1D,0,norm=True) for i in range(d)]
