@@ -26,17 +26,10 @@
 # Author: Daniele Bigoni
 #
 
-"""
-@author: Daniele Bigoni (dabi@imm.dtu.dk)
-
-TensorToolbox is the collection of tools for Tensor decomposition and arithmetic
-
-@copyright: 2014 The Technical University of Denmark
-
-"""
-
 __all__ = []
 
+from _version import __version__
+
 import core
 from core import *
 
