@@ -22,12 +22,11 @@
 # Author: Daniele Bigoni
 #
 
-__all__ = ['STT', 'LINEAR_INTERPOLATION', 'LAGRANGE_INTERPOLATION', 'PROJECTION', 'AVAIL_SURROGATE_TYPES']
+__all__ = ['STT', 'SQTT', 'LINEAR_INTERPOLATION', 'LAGRANGE_INTERPOLATION', 'PROJECTION', 'AVAIL_SURROGATE_TYPES']
 
-import sys
+import logging
 import os.path
 import warnings
-import progressbar
 import time
 import copy
 import random
@@ -39,7 +38,7 @@ from scipy import linalg as scla
 from scipy import sparse as scsp
 from scipy.sparse import linalg as spla
 
-from TensorToolbox.core import TensorWrapper, TTvec, object_store
+from TensorToolbox.core import TensorWrapper, TTvec, QTTvec, storable_object
 from TensorToolbox import multilinalg as mla
 
 from SpectralToolbox import Spectral1D as S1D
@@ -49,7 +48,7 @@ LAGRANGE_INTERPOLATION = 'LagrangeInterpolation'
 PROJECTION    = 'Projection'
 AVAIL_SURROGATE_TYPES = [ LINEAR_INTERPOLATION, LAGRANGE_INTERPOLATION, PROJECTION ]
 
-class STT(object):
+class STT(storable_object):
     """ Constructor of the Spectral Tensor Train approximation. Given a function ``f(x,theta,params):(Is, It) -> R``
     with ``dim(Is)=n`` and ``dim(It)=d``, construct an approximation of ``g(theta,params): It -> h_t(Is)``. For example ``Is`` could be the discretization of a spatial dimension, and ``It`` some parameter space, so that ``f(x,theta,params)`` describes a scalar field depending some parameters that vary in ``It``. The ``params`` in the definition of ``f`` can be constants used by the function or othere objects that must be passed to the function definition.
     
@@ -77,88 +76,96 @@ class STT(object):
     
     """
 
-    TTapprox = None             # np.array of TT approximation
-    TTfour = None               # np.array of TT approximation of the Fourier coefficients
-    Xs_space = None             # List of points (defining the space grid)
-    space_shape = None          # Shape of the space dimension
-    Xs_params = None            # np.array of List of points (defining the parameter grid)
-    Ws = None                   # np.array of List of weights
-
-    marshal_f = None            # Whether to marshal the function or not
-    surrogateONOFF = None       # Whether to construct the surrogate or just the tt-approximation
-    surr_type = None            # Can be Projection or Interpolation
-    orders = None               # np.array of orders for each point
-    poly_types = None           # List of Polynomial types for projection
-    poly_params = None          # List of parameters used in the definition of the Polynomials
-    quad_types = None           # List of quadrature types
-    poly_Xs = None              # List of points used for the construction of the polynomial basis
-    poly_Ws = None              # np.array of List of non-normalized weights
-    Vs = None                   # np.array of List of projection matrices
-    barycentric_weights = None  # np.array of List of barycentric weights for Lagrange Interp
-
-    spans = None                # Span in each dimension
-    range_dim = None            # Number of dimensions of the spatial space
-    param_dim = None            # Number of dimensions of the parameter space
-    TW = None                   # Tensor wrapper containing caching the function evaluations
-    params = None               # Parameters to be passed to f
-
-    stt_store_location = ""     # Location where to store the temp file with computed func values
-    stt_store_freq = None       # Storing frequency
-    
-    eps = None
-    lr_r = None
-    lr_fix_rank = None
-    lr_Jinit = None
-    lr_delta = None
-    lr_maxit = None
-    mv_eps = None
-    mv_maxit = None
-    
-    init = False                # Flags whether the construction of the approximation/surrogate is done.
-
-    start_build_time = None
-    stop_build_time = None
-    last_store_time = None
-
-    serialize_list = ['TTapprox', 'TTfour', 'Xs_space', 'space_shape', 'Xs_params', 'Ws', 'poly_Ws', 'Vs', 'barycentric_weights', 'surrogateONOFF', 'surr_type', 'orders', 'poly_types', 'poly_params', 'quad_types', 'poly_Xs', 'spans', 'range_dim', 'param_dim', 'params', 'stt_store_location', 'stt_store_freq', 'eps', 'lr_r', 'lr_fix_rank', 'lr_Jinit', 'lr_delta', 'lr_maxit', 'mv_eps', 'mv_maxit', 'init','marshal_f']
-    subserialize_list = ['TW']
-    
-    # Parameters to be reset on restarting
-    f = None                    # Function to be approximated
-    polys = None                # List of polynomials (Spectral1D.Poly1D)
+    logger = logging.getLogger(__name__)
+    logger.propagate = False
+    ch = logging.StreamHandler()
+    formatter = logging.Formatter("%(asctime)s %(levelname)s:%(name)s: %(message)s",
+                                  "%Y-%m-%d %H:%M:%S")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
     
     def __init__(self, f, grids, params, range_dim=0, marshal_f=True,
                  surrogateONOFF=False, surrogate_type=None, orders=None, orderAdapt=None, 
-                 eps=1e-4,
-                 lr_r=None, lr_fix_rank=False, lr_Jinit= None, lr_delta=1e-4, lr_maxit=100, 
+                 eps=1e-4, method="ttdmrg",
+                 rs=None, fix_rank=False, Jinit= None, delta=1e-4, maxit=100, 
                  mv_eps=1e-6, mv_maxit = 100,
-                 stt_store_location="",stt_store_overwrite=False, stt_store_freq=0,
-                 maxprocs=None,
-                 empty=False):
-        
-        if empty: return
+                 kickrank = None,
+                 store_location="",store_overwrite=False, store_freq=0):
+
+        super(STT,self).__init__(store_location, store_freq, store_overwrite)
+
+        ##########################################################
+        # List of attributes
+        #
+        self.generic_approx = None       # np.array of TT approximation (can be TTvec or QTTvec)
+        self.TTapprox = None             # np.array of TTvec approximations
+        self.TTfour = None               # np.array of TT approximation of the Fourier coefficients
+        self.Xs_space = None             # List of points (defining the space grid)
+        self.space_shape = None          # Shape of the space dimension
+        self.Xs_params = None            # np.array of List of points (defining the parameter grid)
+        self.Ws = None                   # np.array of List of weights
+
+        self.marshal_f = None            # Whether to marshal the function or not
+        self.surrogateONOFF = None       # Whether to construct the surrogate or just the tt-approximation
+        self.surr_type = None            # Can be Projection or Interpolation
+        self.orders = None               # np.array of orders for each point
+        self.poly_types = None           # List of Polynomial types for projection
+        self.poly_params = None          # List of parameters used in the definition of the Polynomials
+        self.quad_types = None           # List of quadrature types
+        self.poly_Xs = None              # List of points used for the construction of the polynomial basis
+        self.poly_Ws = None              # np.array of List of non-normalized weights
+        self.Vs = None                   # np.array of List of projection matrices
+        self.barycentric_weights = None  # np.array of List of barycentric weights for Lagrange Interp
+
+        self.spans = None                # Span in each dimension
+        self.range_dim = None            # Number of dimensions of the spatial space
+        self.param_dim = None            # Number of dimensions of the parameter space
+        self.TW = None                   # Tensor wrapper containing caching the function evaluations
+        self.params = None               # Parameters to be passed to f
+
+        self.method = None
+        self.eps = None
+        self.rs = None
+        self.fix_rank = None
+        self.Jinit = None
+        self.delta = None
+        self.maxit = None
+        self.mv_eps = None
+        self.mv_maxit = None
+        self.kickrank = None
+
+        self.init = False                # Flags whether the construction of the approximation/surrogate is done.
+
+        self.start_build_time = None
+        self.stop_build_time = None
+
+        # Parameters to be reset on restarting
+        self.f = None                    # Function to be approximated
+        self.polys = None                # List of polynomials (Spectral1D.Poly1D)
+        #
+        # End list of attributes
+        ##########################################################
         
+        self.serialize_list.extend( ['generic_approx', 'TTapprox', 'TTfour', 'Xs_space', 'space_shape', 'Xs_params', 'Ws', 'poly_Ws', 'Vs', 'barycentric_weights', 'surrogateONOFF', 'surr_type', 'orders', 'poly_types', 'poly_params', 'quad_types', 'poly_Xs', 'spans', 'range_dim', 'param_dim', 'params', 'method', 'eps', 'rs', 'fix_rank', 'Jinit', 'delta', 'maxit', 'mv_eps', 'mv_maxit', 'kickrank', 'init','marshal_f'] )
+        self.subserialize_list.extend( ['TW'] )
+
         self.f = f
         self.params = params
         self.marshal_f = marshal_f
         self.surrogateONOFF = surrogateONOFF
         self.surr_type = surrogate_type
-        self.stt_store_location = stt_store_location
-        self.stt_store_freq = stt_store_freq
-        
-        if os.path.isfile(self.stt_store_location) and not stt_store_overwrite:
-            raise AttributeError("The file %s already exist." % self.stt_store_location)
-                
 
-        # Store all the tt-cross parameters
+        # Store all the tt approximation parameters
+        self.method = method
         self.eps = eps
-        self.lr_r = lr_r
-        self.lr_fix_rank = lr_fix_rank
-        self.lr_Jinit = lr_Jinit
-        self.lr_delta = lr_delta
-        self.lr_maxit = lr_maxit
+        self.rs = rs
+        self.fix_rank = fix_rank
+        self.Jinit = Jinit
+        self.delta = delta
+        self.maxit = maxit
         self.mv_eps = mv_eps
         self.mv_maxit = mv_maxit
+        self.kickrank = kickrank
         
         self.range_dim = range_dim
         self.param_dim = len(grids) - self.range_dim
@@ -179,29 +186,25 @@ class STT(object):
                                 self.Xs_params[tuple([0]*max(self.range_dim,1))],\
                                 params = self.params, \
                                 dtype = dtype, \
-                                store_file = self.stt_store_location, store_object = self,
+                                store_file = self.store_location, store_object = self,
                                 marshal_f=marshal_f)
-        
-        # Start construction
-        self.build(maxprocs)
-        
+
     def __getstate__(self):
-        dd = dict( [ (tag, getattr( self, tag )) for tag in self.serialize_list ] )
+        dd = super(STT,self).__getstate__()
         dd['TW'] = self.TW.getstate()
-        self.last_store_time = time.clock()
         return dd
     
     def __setstate__(self,state,f = None):
-        for tag in state.keys():
-            setattr(self, tag, state[tag])
-        
+        super(STT,self).__setstate__( state )
+        # Reset subserialized attributes
         tw = TensorWrapper( None, None, None, empty=True)
         tw.setstate(self.TW)
         self.TW = tw
         
         # Reset additional parameters
         self.set_f(f,self.marshal_f)
-        self.TW.store_object = self
+        self.reset_store_object(self)
+        self.reset_store_time()
         self.polys = [None for i in range(self.param_dim)]
         for i, (poly_type, poly_param) in enumerate(zip(self.poly_types,self.poly_params)):
             if poly_type != None:
@@ -210,19 +213,20 @@ class STT(object):
     def __getitem__(self):
         raise NotImplementedError("To be implemented! (Maybe it's better to just use __call__")
     
-    def __call__(self,x,verbose=False):
-        """ Evaluate the surrogate on points ``x``
+    def __call__(self,x_in,verbose=False):
+        """ Evaluate the surrogate on points ``x_in``
         
-        :param np.ndarray x: 1 or 2 dimensional array of points in the parameter space where to evaluate the function. In 2 dimensions, each row is an entry, i.e. ``x.shape[1] == self.param_dim``
+        :param np.ndarray x_in: 1 or 2 dimensional array of points in the parameter space where to evaluate the function. In 2 dimensions, each row is an entry, i.e. ``x_in.shape[1] == self.param_dim``
         
-        :return: an array with dimension equal to the space dimension (``range_dim``) plus one. If ``A`` is the returned vector and ``range_dim=2``, then ``A[:,:,i]`` is the value of the surrogate for ``x[i,:]``
+        :return: an array with dimension equal to the space dimension (``range_dim``) plus one. If ``A`` is the returned vector and ``range_dim=2``, then ``A[i,:,:]`` is the value of the surrogate for ``x_in[i,:]``
         
         """
         if not (self.init and self.surrogateONOFF):
             raise RuntimeError("The SpectralTensorTrain approximation is not initialized or is not set to construct a surrogate")
         else:
-            if not isinstance(x,np.ndarray) or x.ndim not in [1,2]: raise AttributeError("The input variable must be a 1 or 2 dimensional numpy.ndarray")
-            orig_ndim = x.ndim
+            if not isinstance(x_in,np.ndarray) or x_in.ndim not in [1,2]: raise AttributeError("The input variable must be a 1 or 2 dimensional numpy.ndarray")
+            orig_ndim = x_in.ndim
+            x = x_in.copy()
             if x.ndim == 1:
                 x = np.array([x])
 
@@ -237,9 +241,9 @@ class STT(object):
                         raise AttributeError("The input value " + str(i) + " exceeds the span for dimension " + str(i) )
                     if self.surr_type == PROJECTION:
                         # rescale points to the span [-1,1]
-                        x[:,i] = 2. * (x[:,i] - self.spans[i][0]) / self.spans[i][1] - 1.
+                        x[:,i] = 2. * (x[:,i] - self.spans[i][0]) / (self.spans[i][1] - self.spans[i][0]) - 1.
             
-            output = np.empty(self.space_shape + (Np,))
+            output = np.empty((Np,) + self.space_shape)
 
             mat_cache = [ {} for i in xrange(self.param_dim) ]
             MsI = [ None for i in xrange(self.param_dim) ]
@@ -247,22 +251,18 @@ class STT(object):
             if self.surr_type == PROJECTION: # Projection
                 for ((point,TTf),(_,orders)) in \
                         zip( np.ndenumerate(self.TTfour), np.ndenumerate(self.orders) ):
-                    if verbose:
-                        sys.stdout.write("\033[K Computing point %s\r" % (str(point)))
-                        sys.stdout.flush()
+                    self.logger.debug("Computing point %s" % (str(point)))
                     for i in xrange(self.param_dim):
                         try: MsI[i] = mat_cache[i][orders[i]]
                         except KeyError:
                             mat_cache[i][orders[i]] = self.polys[i].GradVandermonde1D(x[:,i], orders[i], 0, norm=True)
                             MsI[i] = mat_cache[i][orders[i]]
                     TTval = TTf.interpolate(MsI)
-                    output[ point + (slice(None,None,None),) ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
+                    output[ (slice(None,None,None),) + point ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
             elif self.surr_type == LINEAR_INTERPOLATION: # Linear Interpolation
                 for ((point,TTapp),(_,Xs)) in \
                         zip( np.ndenumerate(self.TTapprox),np.ndenumerate(self.Xs_params) ):
-                    if verbose:
-                        sys.stdout.write("\033[K Computing point %s\r" % (str(point)))
-                        sys.stdout.flush()
+                    self.logger.debug("Computing point %s" % (str(point)))
                     for i in xrange(self.param_dim):
                         try: MsI[i] = mat_cache[i][tuple(Xs[i])]
                         except KeyError:
@@ -270,13 +270,11 @@ class STT(object):
                             MsI[i] = mat_cache[i][tuple(Xs[i])]
                     is_sparse = [True] * self.param_dim
                     TTval = TTapp.interpolate(MsI, is_sparse=is_sparse)
-                    output[ point + (slice(None,None,None),) ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
+                    output[ (slice(None,None,None),) + point ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
             elif self.surr_type == LAGRANGE_INTERPOLATION: # Lagrange Interpolation
                 for ((point,TTapp),(_,Xs),(_,bw)) in \
                         zip( np.ndenumerate(self.TTapprox),np.ndenumerate(self.Xs_params),np.ndenumerate(self.barycentric_weights) ):
-                    if verbose:
-                        sys.stdout.write("\033[K Computing point %s\r" % (str(point)))
-                        sys.stdout.flush()
+                    self.logger.debug("Computing point %s" % (str(point)))
                     for i in xrange(self.param_dim):
                         try: MsI[i] = mat_cache[i][tuple(Xs[i])]
                         except KeyError:
@@ -284,22 +282,20 @@ class STT(object):
                             MsI[i] = mat_cache[i][tuple(Xs[i])]
                     is_sparse = [False] * self.param_dim
                     TTval = TTapp.interpolate(MsI, is_sparse=is_sparse)
-                    output[ point + (slice(None,None,None),) ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
+                    output[ (slice(None,None,None),) + point ] = np.asarray( [ TTval[tuple([i]*self.param_dim)] for i in range(Np) ] ) 
             else:
                 raise AttributeError("Type of interpolation not defined")
 
-            if verbose:
-                sys.stdout.write("\033[K Computing DONE\n")
-                sys.stdout.flush()
+            self.logger.debug("Computing DONE")
 
             if orig_ndim == 1:
                 if self.range_dim == 0:
-                    return output[tuple( [slice(None,None,None)]*self.range_dim ) + (0,)][0]
+                    return output[ (0,) + tuple( [slice(None,None,None)]*self.range_dim )][0]
                 else:
-                    return output[tuple( [slice(None,None,None)]*self.range_dim ) + (0,)]
+                    return output[ (0,) + tuple( [slice(None,None,None)]*self.range_dim )]
             else:
                 if self.range_dim == 0:
-                    return output[0,:]
+                    return output[:,0]
                 else:
                     return output
 
@@ -397,6 +393,31 @@ class STT(object):
     
     def setstate(self,state,f = None):
         self.__setstate__(state, f)
+
+    def h5store(self, h5file):
+        self.TW.h5store(h5file)
+    
+    def h5load(self, h5file):
+        self.TW.h5load(h5file)
+    
+    def to_v_0_3_0(self, store_location):
+        """ Upgrade to v0.3.0
+        
+        :param string filename: path to the filename. This must be the main filename with no extension.
+        """
+        super(STT,self).to_v_0_3_0(store_location)
+        # Call the upgrade for the Tensor Wrapper
+        self.TW.to_v_0_3_0(store_location)
+        # Call the upgrade for all the TTapproximations
+        for (_,tt) in np.ndenumerate(self.TTapprox):
+            if tt != None:
+                tt.to_v_0_3_0(store_location)
+        for (_,tt) in np.ndenumerate(self.TTfour):
+            if tt != None:
+                tt.to_v_0_3_0(store_location)
+        for (_,tt) in np.ndenumerate(self.generic_approx):
+            if tt != None:
+                tt.to_v_0_3_0(store_location)
     
     def copy(self):
         C = copy.deepcopy(self)
@@ -409,8 +430,21 @@ class STT(object):
         return C
  
     def set_f(self,f,marshal_f=True):
-        self.f = f
-        self.TW.set_f(self.f,marshal_f)
+        if f == None:
+            self.TW.reset_f_marshal()
+        else:
+            self.TW.set_f(f,marshal_f)
+        self.f = self.TW.f
+    
+    def set_params(self, params):
+        self.params = params
+        self.TW.set_params(params)
+
+    def reset_store_object(self, obj):
+        self.set_store_object( None )
+        self.TW.set_store_object( obj )
+        for _, TTapp in np.ndenumerate(self.generic_approx):
+            if TTapp != None: TTapp.set_store_object(obj)
 
     def set_grids(self,grids,orders=None):
         # Store grid for spatial space
@@ -481,107 +515,107 @@ class STT(object):
                     self.Ws[point][i] = w
                 else:
                     raise AttributeError("The %d argument of grid is none of the types accepted." % i)
-        
+    
+    def empty_generic_approx(self):
+        self.generic_approx = np.empty(self.space_shape, dtype=TTvec)
+    
+    def new_generic_approx(self,multidim_point):
+        return TTvec( self.TW,
+                      store_location=self.store_location,
+                      store_freq=self.store_freq,
+                      store_object=self,
+                      multidim_point=multidim_point )
 
     def build(self,maxprocs=None,force_redo=False):
         self.start_build_time = time.clock()
-        self.last_store_time = self.start_build_time
         
-        if self.TTapprox == None or force_redo:
-            self.TTapprox = np.empty(self.space_shape, dtype=TTvec)
+        if self.generic_approx == None or force_redo:
+            self.empty_generic_approx()
         
         self.TW.set_maxprocs(maxprocs) # Set the number of processors for MPI
-        for point, val in np.ndenumerate(self.TTapprox):
+        for point, val in np.ndenumerate(self.generic_approx):
             if val == None or not val.init:
                 try:
                     totsize = float(self.TW.get_size())
-                    print "Point %s - Fill: %f%%" % (str(point),float(self.TW.get_fill_level())/totsize * 100.)
+                    self.logger.info( "Point %s - Fill: %f%%" % (str(point),float(self.TW.get_fill_level())/totsize * 100.) )
                 except OverflowError:
-                    print "Point %s - Fill: %d" % (str(point),self.TW.get_fill_level())
+                    self.logger.info( "Point %s - Fill: %d" % (str(point),self.TW.get_fill_level()) )
                 multidim_point = point if self.range_dim > 0 else None
-                # Build TTapprox for the selected point
-                if val == None or self.TTapprox[point].ttcross_Jinit == None:
+                # Build generic_approx for the selected point
+                if val == None or self.generic_approx[point].Jinit == None:
                     # Find all the back neighbors and select the first found to start from
                     neigh = None
                     for i in range(self.range_dim-1,-1,-1): 
                         if point[i]-1 >= 0:
                             pp = list(point)
                             pp[i] -= 1
-                            neigh = self.TTapprox[tuple(pp)]
+                            neigh = self.generic_approx[tuple(pp)]
                             break
                     # If a neighbor is found, select the rank and the fibers to be used
                     if neigh != None:
-                        rs = neigh.ranks()
-                        for i in range(1,len(rs)-1):
-                            rs[i] += 1
-                        Js = neigh.ttcross_Js_last[-2]
-                        # Trim the fibers according to rs (This allow the rank to decrease as well)
-                        for r,(j,J) in zip(rs[1:-1],enumerate(Js)):
-                            Js[j] = random.sample(J,r)
-                        self.lr_r = rs
-                        self.lr_Jinit = Js
+                        if self.method == 'ttcross':
+                            rs = neigh.ranks()
+                            for i in range(1,len(rs)-1):
+                                rs[i] += 1
+                            Js = neigh.Js_last[-2]
+                            # Trim the fibers according to rs (This allow the rank to decrease as well)
+                            for r,(j,J) in zip(rs[1:-1],enumerate(Js)):
+                                Js[j] = random.sample(J,r)
+                            self.rs = rs
+                            self.Jinit = Js
+                        elif self.method == 'ttdmrg':
+                            self.Jinit = neigh.Jinit
+                            
 
-                    self.TTapprox[point] = TTvec( self.TW, generate=False )
-                    self.TTapprox[point].outer_ttcross(eps=self.eps,
-                                                       lr_r=self.lr_r, 
-                                                       fix_rank=self.lr_fix_rank, 
-                                                       lr_Jinit= self.lr_Jinit, 
-                                                       lr_delta=self.lr_delta, lr_maxit=self.lr_maxit, 
-                                                       mv_eps=self.mv_eps, mv_maxit = self.mv_maxit,
-                                                       lr_store_location=self.stt_store_location, store_object=self, store_freq=self.stt_store_freq,
-                                                       multidim_point=multidim_point)
+                    self.generic_approx[point] = self.new_generic_approx(multidim_point)
+                    self.generic_approx[point].build(eps=self.eps, method=self.method,
+                                                     rs=self.rs, 
+                                                     fix_rank=self.fix_rank, 
+                                                     Jinit= self.Jinit, 
+                                                     delta=self.delta, maxit=self.maxit, 
+                                                     mv_eps=self.mv_eps, mv_maxit = self.mv_maxit,
+                                                     kickrank=self.kickrank)
                 else:           # Use stored Jinit to restart
-                    Jinit = self.TTapprox[point].ttcross_Jinit
-                    lr_r = [ len(Jinit[i]) for i in range(len(self.TTapprox[point].ttcross_Jinit)) ]
-                    lr_r.insert(0,1)
-                    self.TTapprox[point].A = self.TW
-                    self.TTapprox[point].outer_ttcross(eps=self.eps,
-                                                       lr_r=lr_r, 
-                                                       fix_rank=self.lr_fix_rank, 
-                                                       lr_Jinit= Jinit, 
-                                                       lr_delta=self.lr_delta, lr_maxit=self.lr_maxit, 
-                                                       mv_eps=self.mv_eps, mv_maxit = self.mv_maxit,
-                                                       lr_store_location=self.stt_store_location, store_object=self, store_freq=self.stt_store_freq,
-                                                       multidim_point=multidim_point)
+                    Jinit = self.generic_approx[point].Jinit
+                    rs = [ len(Jinit[i]) for i in range(len(self.generic_approx[point].Jinit)) ]
+                    rs.insert(0,1)
+                    self.generic_approx[point].A = self.TW
+                    self.generic_approx[point].build(eps=self.eps, method=self.method,
+                                                     rs=rs, 
+                                                     fix_rank=self.fix_rank, 
+                                                     Jinit=Jinit, 
+                                                     delta=self.delta, maxit=self.maxit, 
+                                                     mv_eps=self.mv_eps, mv_maxit = self.mv_maxit,
+                                                     kickrank=self.kickrank)
 
-                # # Store Jinit for next point (Use the last used values)
-                # self.lr_Jinit = self.TTapprox[point].ttcross_Js
-                # self.lr_r = self.TTapprox[point].ttcross_rs
+                self.store()
 
-                # # Store Jinit for next point (Use the second to last used values)
-                # self.lr_Jinit = self.TTapprox[point].ttcross_Js_last[-2]
-                # self.lr_r = self.TTapprox[point].ttcross_rs
+        self.store(force=True)
 
-                if self.to_be_stored(): object_store(self.stt_store_location, self)
-            else:
-                # # Get Jinit for next point
-                # self.lr_Jinit = self.TTapprox[point].ttcross_Js
-                # self.lr_r = self.TTapprox[point].ttcross_rs
-
-                # Store Jinit for next point (Use the second to last used values)
-                self.lr_Jinit = self.TTapprox[point].ttcross_Js_last[-2]
-                self.lr_r = self.TTapprox[point].ttcross_rs
-
-        if self.stt_store_location != None and \
-                self.stt_store_location != "":
-            object_store(self.stt_store_location,self)
+        self.prepare_TTapprox()
+        
+        self.store()
         
         self.prepare_surrogate()
         
         self.init = True
 
-        if self.stt_store_location != None and \
-                self.stt_store_location != "":
-            object_store(self.stt_store_location,self)
+        self.store(force=True)
 
         self.stop_build_time = time.clock()
     
-    def to_be_stored(self):
-        if self.stt_store_location != '' and \
-                time.clock() > self.last_store_time + self.stt_store_freq: 
-            return True
-        else: return False
-    
+    def prepare_TTapprox(self, force_redo=False):
+        """ Prepares the TTapprox from the generic_approx
+        """
+        if self.TTapprox == None or force_redo:
+            self.TTapprox = np.empty(self.space_shape, dtype=TTvec)
+        
+        for (point,gen_app) in np.ndenumerate(self.generic_approx):
+            self.TTapprox[point] = self.generic_to_TTvec( gen_app )
+
+    def generic_to_TTvec(self, gen_app):
+        return gen_app
+
     def prepare_surrogate(self, force_redo=False):
         if self.surrogateONOFF and self.surr_type == PROJECTION:
             self.build_projection_surrogate(force_redo)
@@ -597,11 +631,9 @@ class STT(object):
         for (point, val),(_,ttapp) in zip(np.ndenumerate(self.TTfour), np.ndenumerate(self.TTapprox)) :
             if val == None and ttapp != None:
                 self.Vs[point] = [ self.polys[i].GradVandermonde1D(self.poly_Xs[point][i], self.orders[point][i], 0, norm=True) for i in range(self.param_dim) ]
-                # sys.stdout.write("\033[K Projecting point %s\r" % (str(point)))
-                # sys.stdout.flush()
+                self.logger.debug("Projecting point %s" % (str(point)))
                 self.TTfour[point] = ttapp.project(self.Vs[point],self.poly_Ws[point])
-        # sys.stdout.write("\033[K Projection Done\n")
-        # sys.stdout.flush()
+        self.logger.debug("Projection Done")
     
     def build_barycentric_weights(self, force_redo=False):
         if self.barycentric_weights == None or force_redo:
@@ -628,3 +660,77 @@ class STT(object):
             if self.range_dim == 0: return output[0]
             else: return output
             
+
+
+class SQTT(STT):
+    """ Constructor of the Spectral Quantics Tensor Train approximation. Given a function ``f(x,theta,params):(Is, It) -> R``
+    with ``dim(Is)=n`` and ``dim(It)=d``, construct an approximation of ``g(theta,params): It -> h_t(Is)``. For example ``Is`` could be the discretization of a spatial dimension, and ``It`` some parameter space, so that ``f(x,theta,params)`` describes a scalar field depending some parameters that vary in ``It``. The ``params`` in the definition of ``f`` can be constants used by the function or othere objects that must be passed to the function definition.
+    
+    :param function f: multidimensional function to be approximated with format ``f(x,theta,params)``
+    :param list grids: this is a list with ``len(grids)=dim(Is)+dim(It)`` which can contain:
+      a) 1-dimensional numpy.array of points discretizing the i-th dimension,
+      b) a tuple ``(PolyType,QuadType,PolyParams,span)`` where ``PolyType`` is one of the polynomials available in :py:mod:`SpectralToolbox.Spectral1D` and ``QuadType`` is one of the quadrature rules associated to the selected polynomial and ``PolyParams`` are the parameters for the selected polynomial. ``span`` is a tuple defining the left and right end for dimension i (Example: ``(-3,np.inf)``)
+      c) a tuple ``(QuadType,span)`` where ``QuadType`` is one of the quadrature rules available in :py:mod:`SpectralToolbox.Spectral1D` without the selection of a particular polynomial type, and ``span`` is defined as above.
+    :param object params: any list of parameters to be passed to the function ``f``
+    :param int range_dim: define the dimension of the spatial dimension ``Is``. For functionals ``f(theta,params)``, ``dim(Is)=0``. For scalar fileds in 3D, ``dim(Is)=3``.
+    :param bool marshal_f: whether to marshal the function f or not. For MPI support, the function f must be marshalable (does this adverb exists??).
+    :param int base: ``base`` parameter for Quantics Tensor Train
+    :param bool surrogateONOFF: whether to construct the surrogate or not
+    :param str surrogate_type: whether the surrogate will be an interpolating surrogate (``TensorTrain.LINEAR_INTERPOLATION`` or ``TensorTrain.LAGRANGE_INTERPOLATION``) or a projection surrogate (``TensorTrain.PROJECTION``)
+    :param list orders: polynomial orders for each dimension if ``TensorTrain.PROJECTION`` is used. If ``orderAdapt==True`` then the ``orders`` are starting orders that can be increased as needed by the construction algorithm. If this parameter is not provided but ``orderAdapt==True``, then the starting order is 1 for all the dimensions.
+    :param bool orderAdapt: whether the order is fixed or not.
+    :param str stt_store_location: path to a file where function evaluations can be stored and used in order to restart the construction.
+    :param bool stt_store_overwrite: whether to overwrite pre-existing files
+    :param int stt_store_freq: storage frequency. Determines every how many seconds the state is stored. ``stt_store_freq==0`` stores every time it is possible.
+    :param bool empty: Creates an instance without initializing it. All the content can be initialized using the ``setstate()`` function.
+
+    .. note:: For a description of the remaining parameters see :py:class:`TTvec`.
+    .. document private functions
+    .. automethod:: __getitem__
+    .. automethod:: __call__
+    
+    """
+
+    def __init__(self, f, grids, params, range_dim=0, marshal_f=True,
+                 base = 2,
+                 surrogateONOFF=False, surrogate_type=None, orders=None, orderAdapt=None, 
+                 eps=1e-4, method="ttdmrg",
+                 rs=None, fix_rank=False, Jinit= None, delta=1e-4, maxit=100, 
+                 mv_eps=1e-6, mv_maxit = 100,
+                 kickrank = None,
+                 store_location="",store_overwrite=False, store_freq=0):
+        
+        super(SQTT,self).__init__(f, grids, params, range_dim=range_dim, marshal_f=marshal_f,
+                                  surrogateONOFF=surrogateONOFF, surrogate_type=surrogate_type, orders=orders, orderAdapt=orderAdapt, 
+                                  eps=eps, method=method,
+                                  rs=rs, fix_rank=fix_rank, Jinit=Jinit, delta=delta, maxit=maxit, 
+                                  mv_eps=mv_eps, mv_maxit=mv_maxit,
+                                  kickrank=kickrank,
+                                  store_location=store_location,store_overwrite=store_overwrite, store_freq=store_freq)
+
+        ##########################################################
+        # List of attributes
+        #
+        self.base = None
+        
+        #
+        # End list of attributes
+        ##########################################################
+        
+        self.serialize_list.extend( ['base'] )
+        
+        self.base = base
+
+    
+    def empty_generic_approx(self):
+        self.generic_approx = np.empty(self.space_shape, dtype=QTTvec)
+    
+    def new_generic_approx(self,multidim_point):
+        return QTTvec( self.TW,
+                       store_location=self.store_location,
+                       store_freq=self.store_freq,
+                       store_object=self,
+                       multidim_point=multidim_point )
+    
+    def generic_to_TTvec(self, gen_app):
+        return gen_app.to_TTvec()
