@@ -26,8 +26,8 @@
 # Author: Daniele Bigoni
 #
 
-import aux
-from aux import *
+import auxiliary
+from auxiliary import *
 import storage
 from storage import *
 import tensor_wrapper
@@ -46,7 +46,7 @@ import SpectralTensorTrain
 from SpectralTensorTrain import *
 
 __all__ = []
-__all__ += aux.__all__
+__all__ += auxiliary.__all__
 __all__ += storage.__all__
 __all__ += tensor_wrapper.__all__
 __all__ += candecomp.__all__
